package com.kernelmaft.zanbur

import kotlinx.coroutines.Dispatchers.IO
import kotlinx.coroutines.flow.firstOrNull
import kotlinx.coroutines.launch
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.getValue
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.datastore.preferences.core.edit
import androidx.datastore.preferences.core.stringPreferencesKey
import androidx.lifecycle.lifecycleScope


class MainActivity : ComponentActivity () {
	override fun onCreate ( savedInstanceState : Bundle ? ) {
		super . onCreate (savedInstanceState)

		var groups by AppState . groups
		groups = Config . groups

		lifecycleScope . launch (IO) {
			val prefs = applicationContext . dataStore . data . firstOrNull ()
			if ( prefs != null ) {
				val savedSceneName = prefs [ stringPreferencesKey ("scene") ]
				if ( savedSceneName != null ) {
					val savedScene = groups [0] . scenes . find { it . name == savedSceneName }
					if ( savedScene != null ) {
						AppState . setCurrentScene ( 0 , savedScene )
					}
				}
			}
		}

		enableEdgeToEdge ()
		setContent {
			ZanburTheme {
				Scaffold { scaffoldPadding ->
					CenteredColumn ( Modifier . padding (scaffoldPadding) . fillMaxSize () ) {

						CenteredColumn ( Modifier . width ( 300 . dp ) ) {
							Text (groups [0] . name)
							Spacer ( Modifier . height ( compactSpacing ) )
							SceneSwitcher ( groups [0] . scenes , groups [0] . currentScene ) {
								AppState . setCurrentScene ( 0 , it )
								publishSceneChange ( groups [0] , it )
							}
						}
					}
				}
			}
		}

		MqttClient . run (lifecycleScope)
	}

	override fun onStop () {
		super . onStop ()

		val groups by AppState . groups
		val currentScene = groups [0] . currentScene

		if ( currentScene != null ) lifecycleScope . launch (IO) {
			applicationContext . dataStore . edit {
				it [ stringPreferencesKey ("scene") ] = currentScene . name
			}
		}
	}
}
